#!/bin/bash
#Copyright [2018 - 2020] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

#Move elastic 7.7 to old elastic search

TMP_LOG_FILE="/tmp/cis_elastic_prepare.log"
DATE_NOW=$(date +%s)
LOG_FILE="/var/opt/novell/log/cis/cis_elastic_prepare_$DATE_NOW.log"

printandExit(){
  if [[ $1 -ne 0 ]]; then
    echo -e "Elastic service update failed." | tee -a "$TMP_LOG_FILE"
    mv "$TMP_LOG_FILE" "$LOG_FILE"
    exit 1
  else
    echo -e "Elastic service updated successfully." | tee -a "$TMP_LOG_FILE"
    mv "$TMP_LOG_FILE" "$LOG_FILE"
    exit 0
  fi
}

isElastic7_7() {
    elasticversion=""
    
    configFile="/opt/novell/cis/es/bin/es_service.sh"
    line='${ES_NAME} docker.elastic.co/elasticsearch/elasticsearch-oss:7.7.0 &'
    result=`grep "$line" "$configFile"`

    if [ -f $configFile ]; then
        echo "es_service file exists. " >> "$TMP_LOG_FILE" 2>&1
        if [ -z "$result" ]; then
            echo "Not a 7.7 install " >> "$TMP_LOG_FILE" 2>&1
            printandExit 0            
        else 
            echo "It is a 7.7 install " >> "$TMP_LOG_FILE" 2>&1            
            elasticversion="7.7.0"            
        fi
    else
        echo "es_service file is not accessible " >> "$TMP_LOG_FILE" 2>&1        
    fi

    if [ -d /etc/opt/novell/cis/es ]; then
        echo "Directory /etc/opt/novell/cis/es is available " >> "$TMP_LOG_FILE" 2>&1
    else 
        echo "Directory /etc/opt/novell/cis/es is unavailable " >> "$TMP_LOG_FILE" 2>&1        
    fi

    if [ "$elasticversion" == "7.7.0" ]; then
        return 1
    else 
        return 0
    fi
}

prepare() {
    isElastic7_7
    
    if [ "$?" == 0 ]; then
        echo "It is not a 7.7.0 install. Exiting." >> "$TMP_LOG_FILE" 2>&1
        printandExit 0
    else 
        if [ ! -d /opt/novell/cis/oldes ]; then  
            echo  "Creating opt dir " >>  "$TMP_LOG_FILE" 2>&1
            # move all older elastic to different folders  
            cp /var/opt/novell/cis/elastic.tar.gz /var/opt/novell/oldcis  

            mkdir /opt/novell/cis/oldes
            cp -r /opt/novell/cis/es/* /opt/novell/cis/oldes

            sed -i -e"s/#precheck/sleep 5 #precheck/g" /opt/novell/cis/oldes/bin/es_service.sh   
            sed -i -e"s/\/etc\/opt\/novell\/cis\/es/\/etc\/opt\/novell\/cis\/oldes/g" /opt/novell/cis/oldes/bin/es_service.sh 
            sed -i -e"s/\/var\/opt\/novell\/cis/\/var\/opt\/novell\/oldcis/g" /opt/novell/cis/oldes/bin/es_service.sh 
            sed -i -e"s/\/cis\/es\/bin\/cis_enable_firewall.sh elastic/\/cis\/oldes\/bin\/cis_enable_firewall.sh oldelastic/g" /opt/novell/cis/oldes/bin/es_service.sh 
            sed -i -e"s/9300:9300/19300:9300/g" /opt/novell/cis/oldes/bin/es_service.sh 
            #remove docker rmi 
            sed -i -e"s/docker rmi/#docker rmi/g" /opt/novell/cis/oldes/bin/es_service.sh   

            sed -i -e"s/exit 0/logger -t cis \"NOOP.\"/g" /opt/novell/cis/oldes/bin/cis_enable_firewall.sh
            sed -i -e"s/rpm -q --queryformat/logger -t cis \"NOOP.\"#rpm -q --queryformat/g" /opt/novell/cis/oldes/bin/cis_enable_firewall.sh
            
            # if it is not media
            if { [ -L /etc/opt/novell/cis ] && [ -e /etc/opt/novell/cis ]; };then          
                 createOldES $TMP_LOG_FILE
            fi     
        fi   
    fi
    printandExit 0
}

createOldES() {  
    TMP_LOG_FILE=""

    if [ "$1" == "" ]; then
        DATE_NOW=$(date +%s)
        TMP_LOG_FILE="/var/opt/novell/log/cis/cis_elastic_prepare_$DATE_NOW.log"
    else
        TMP_LOG_FILE="$1"
    fi

    if { [ ! -d /etc/opt/novell/cis/oldes ] &&  [ -d /opt/novell/cis/oldes ]; }; then
        echo  "Creating etc dir " >>  "$TMP_LOG_FILE" 2>&1
        mkdir /etc/opt/novell/cis/oldes
        mkdir /etc/opt/novell/cis/oldes/data    
        mv /etc/opt/novell/cis/es/data/* /etc/opt/novell/cis/oldes/data
        /usr/bin/chmod 777 -R /etc/opt/novell/cis/oldes/data
        /usr/bin/chmod 777 -R /etc/opt/novell/cis/es/data

        if [ -f /etc/opt/novell/cis/oldes/data/nodes/0/node.lock ]; then
            rm /etc/opt/novell/cis/oldes/data/nodes/0/node.lock        
        fi
        if [ -f /etc/opt/novell/cis/oldes/data/nodes/0/_state/write.lock ]; then
            rm /etc/opt/novell/cis/oldes/data/nodes/0/_state/write.lock
        fi

        cp /etc/opt/novell/cis/es/config /etc/opt/novell/cis/oldes
        if [ -f /etc/opt/novell/cis/es/VERSION ]; then
            cp /etc/opt/novell/cis/es/VERSION /etc/opt/novell/cis/oldes
        fi
        cp /etc/opt/novell/cis/es/es_envoy.json /etc/opt/novell/cis/oldes
        cp /etc/opt/novell/cis/es/log4j2.properties /etc/opt/novell/cis/oldes

        sed -i -e"s/cis-es/cis-old-es/g" /etc/opt/novell/cis/oldes/config
        sed -i -e"s/\/es\//\/oldes\//g" /etc/opt/novell/cis/oldes/config
        sed -i -e"s/9400/9500/g" /etc/opt/novell/cis/oldes/config

        sed -i -e"s/9400/9500/g" /etc/opt/novell/cis/oldes/es_envoy.json
    fi
    
}

enableservice() {
    # Do not enable elastic service if already sp2
    isElastic7_7
    
    if [ "$?" == 0 ]; then        
        printandExit 0
    fi
    
    config_Type=`cat /etc/opt/novell/cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'configType'\042/){print $(i+1)}}}' | tr -d '"' | xargs`
    if { [ $config_Type=="standalone" ] && [ -d /var/opt/novell/oldcis ]; }; then
        sed -i 's/9092 9400/9092 9400 9500 /g' /etc/sysconfig/SuSEfirewall2
        rcSuSEfirewall2 restart      

        systemctl enable oes-cis-old-elastic.service
        printandExit 0
    fi
}

cleanup() {
    config_Type=`cat /etc/opt/novell/cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'configType'\042/){print $(i+1)}}}' | tr -d '"' | xargs`
    MOVE_LOG_FILE="$1"

    if [ $config_Type == "standalone" ]; then
        systemctl stop oes-cis-old-elastic.service
        if [ $? -ne 0 ]; then 
                echo "Could not stop Elastic 7.7 service " >> "$MOVE_LOG_FILE" 2>&1         
                exit 1
        fi    
        systemctl disable oes-cis-old-elastic.service >> "$MOVE_LOG_FILE" 2>&1        
        
    fi   

    cleanupRemaining "$MOVE_LOG_FILE"
}

cleanupRemaining() {
    MOVE_LOG_FILE="$1"
    if [ "$MOVE_LOG_FILE" == "" ]; then
       MOVE_LOG_FILE=/tmp/ciselastic 
    fi

    rm -rf /opt/novell/cis/oldes
    rm -rf /etc/opt/novell/cis/oldes
    rm -rf /var/opt/novell/oldcis

    rm /usr/lib/systemd/system/oes-cis-old-elastic.service >> "$MOVE_LOG_FILE" 2>&1  
    systemctl daemon-reload   

    #remove docker images
    docker rmi $(docker images --filter "reference=docker.elastic.co/elasticsearch/elasticsearch-oss" --filter "before=docker.elastic.co/elasticsearch/elasticsearch-oss:6.8.22" -q) >> "$MOVE_LOG_FILE" 2>&1
    docker rmi elasticdump/elasticsearch-dump >> "$MOVE_LOG_FILE" 2>&1
}

loadDump(){
    MOVE_LOG_FILE=$1
    /usr/bin/docker load -i /var/opt/novell/cis/elasticdump.tar.gz >> "$MOVE_LOG_FILE" 2>&1 
}

startDump(){    
    MOVE_LOG_FILE=$1
    config_Type=`cat /etc/opt/novell/cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'configType'\042/){print $(i+1)}}}' | tr -d '"' | xargs`

    if [ $config_Type == "standalone" ]; then
        systemctl status oes-cis-elastic.service >> "$MOVE_LOG_FILE" 2>&1   
        if [ $? -ne 0 ]; then 
            echo "Elastic service is not running" >> "$MOVE_LOG_FILE" 2>&1         
            exit 1
        fi
        systemctl status oes-cis-old-elastic.service >> "$MOVE_LOG_FILE" 2>&1  
        if [ $? -ne 0 ]; then 
            echo "Elastic service 7.7 is not running" >> "$MOVE_LOG_FILE" 2>&1                        
            exit 1
        fi    
    fi

    # read data from config file
    BATCH=`grep "BATCH" /etc/opt/novell/cis/es/dumpconfig | cut -d '=' -f 2`
    CONCURRENCY=`grep "CONCURRENCY" /etc/opt/novell/cis/es/dumpconfig | cut -d '=' -f 2`

    ipconfig=`grep IP_CONFIG /etc/opt/novell/cis/config`
    localip=`echo $ipconfig | cut -d '"' -f2`

    getDocumentCount
    count=$(cat /tmp/ciselasticcount)
    echo "Number of documents to be migrated " $count >> "$MOVE_LOG_FILE" 2>&1  

    if [ $count != "-1" ]; then
        /usr/bin/docker run --rm -i -v /etc/opt/novell/cis/certs:/etc elasticdump/elasticsearch-dump:latest \
        --input https://$localip:9500 --output https://$localip:9400 --type=data --cert /etc/servercert.pem \
        --key /etc/serverkey.pem --ca /etc/rootCAs/SSCert.pem --tlsAuth --delete --support-big-int --limit ${BATCH} --concurrency ${CONCURRENCY} >> "$MOVE_LOG_FILE" 2>&1    
        
        # check if elastic copying is complete
        count="1"    
        iteration=1
            
        while [ $iteration -lt 5 ];
        do            
            getDocumentCount        
            count=$(cat /tmp/ciselasticcount)
            echo "Getting count " $count " " $iteration >> "$MOVE_LOG_FILE" 2>&1        
            if { [ $count == "-1" ] || [ $count == "0" ]; }; then 
                break
            fi
            iteration=$[iteration + 1]
        done

        if [ $count == "0" ]; then 
            echo "Elastic data has migrated. " >> "$MOVE_LOG_FILE" 2>&1  
            echo "Cleaning up" >> "$MOVE_LOG_FILE" 2>&1          
            cleanup $MOVE_LOG_FILE
            echo "Elastic data migrated sucessfully. " | tee -a "$MOVE_LOG_FILE"
            exit 0
        else             
            PrintUnsuccessful $config_Type $MOVE_LOG_FILE
        fi
    else
        PrintUnsuccessful $config_Type $MOVE_LOG_FILE
    fi    
}

PrintUnsuccessful() {
    MOVE_LOG_FILE=$2
    echo "Elastic data has not migrated sucessfully. Please run the below command again." | tee -a "$MOVE_LOG_FILE"  
    if [ $1 == "standalone" ]; then
        echo "sh /var/opt/novell/cis/elastic_move_data.sh migrate" | tee -a "$MOVE_LOG_FILE" 
    else
        echo "sh /opt/novell/cis/bin/cis_ext_service.sh migrate" | tee -a "$MOVE_LOG_FILE" 
    fi
    exit 1
}

stopDump() {
    MOVE_LOG_FILE=$1
    /usr/bin/docker stop elasticdump/elasticsearch-dump >> "$MOVE_LOG_FILE" 2>&1  
}

getDocumentCount() {
    ipconfig=`grep IP_CONFIG /etc/opt/novell/cis/config`
    localip=`echo $ipconfig | cut -d '"' -f2`

    secure=`grep "SECURE_MODE" /etc/opt/novell/cis/es/config | cut -d '=' -f 2`
    /opt/novell/cis/bin/cismigrate --count --zk_uri $localip --secure $secure
}

migrate() {
    DATE_NOW=$(date +%s)
    MOVE_LOG_FILE="/var/opt/novell/log/cis/cis_elastic_move_$DATE_NOW.log"

    loadDump $MOVE_LOG_FILE
    startDump $MOVE_LOG_FILE

    if [ $?==0] ; then
        exit 0
    else 
        exit 1
    fi
}

case "$1" in
    startDump)
        startDump
        ;;
    stopDump)
        stopDump
        ;;
	loadDump)
		loadDump
		;;
    prepare)
        prepare
        ;;
    enableservice)
        enableservice
        ;;
    createOldES)
        createOldES
        ;;   
    getDocumentCount)
        getDocumentCount
        ;;
    migrate)
        migrate
        ;;
    cleanupRemaining)
        cleanupRemaining
        ;;
esac


